#include <stdlib.h>
#include <stdio.h>
#include "types.h"
#include "proto.h"
#include "select.h"
#include "matrix.h"
#include "print.h"
#include "read.h"
#include "transform.h"
#include "statistics.h"
#include "plot.h"
#include "menu.h"

/* local definitions */
typedef \
   enum { AVER_ERROR, AVER_NONE, AVER_BY_PARTI, AVER_BY_QUERY, AVER_BY_BOTH } \
average_type;

/* local prototypes */
static void descrip_menu(const chose_t * parti_s, const chose_t * query_s, 
         chose_t * level_s[], Matrix ** data[], Matrix **chosen[], 
         char level_list[], string tran_str);
static void level_menu(chose_t * level_select[], char level_list[]);
static void anova_menu(const bool inter_results, const string level_list,
         const chose_t * parti_s, const chose_t * query_s, 
         chose_t * level_s[], Matrix ** data[], Matrix ** chosen[], 
         char trans_str[]);
static void options_menu(bool *inter_results, char transform_str[]);
static void data_menu(chose_t * parti_s, chose_t * queri_s);
static void file_menu(Matrix * prec[], Matrix * interp[], Matrix * numdoc[], 
         chose_t * parti_s, chose_t * query_s);
static bool is_3way(chose_t * level_s[]);
static bool prep_RP_plot(const chose_t * parti_s, const chose_t * query_s, 
         Matrix ** all_data[], Matrix ** interp_only[], string parti_name[],
         real * average[NUMofINTERPOL]);
static char prep_histo(const string parti_name[], const string query_name[], 
         chose_t * level_s[], const average_type ave_status,
         Matrix ** chosen[], string label[MAX_HISTO_LABEL], 
         real * toprint[NUMofPRECISION], matrix_dim *prec);
static bool histo_labels(string label[], const string parti_name[], 
         const string query_name[], const matrix_dim max_parti, 
         const matrix_dim max_query, const average_type ave_status);
static void misc_menu(void);

/* module global variables */
   static const string   no_level[]   = {NULL};
   static const string   inter_name[] = {"0.0", "0.1", "0.2", "0.3", "0.4",
                                         "0.5", "0.6", "0.7", "0.8", "0.9",
                                         "1.0"};
   static const string   nd_name[]    = {   "5",  "10",   "15",   "20", "30", 
                                          "100", "200",  "500", "1000"};
   static const elem_t   typelen[]    = {0, 0, 3, 4};

/* global variables exported to other modules */
          const string   typename[]    = {"Average precision", 
                                          "R-precision",
                                          "Precision at fixed recall level",
                                          "Precision at fixed numbers of "
                                             "retrieved documents"};
          const string *   valname[]   = {no_level,   no_level, 
                                          inter_name, nd_name};
          const elem_t     typesize[]  = {1, 1, NUMofINTERPOL, NUMofNUMDOCS};

/* global variables imported from other modules */
   extern const char       product[];  /* from main.c */
   extern const char       version[];  /* from main.c */

/*                                                                         *\
   -----------------------------------------------------------------------
                                   Menus
   -----------------------------------------------------------------------
\*                                                                         */

bool main_menu(Matrix *  precision_mat[],
               Matrix *  interpolate_mat[], 
               Matrix *  numdoc_mat[],
               chose_t * participant_s,
               chose_t * query_s)
{
            const char           proc[] = "main_menu";
   extern   const char           PROMPT[];
   extern         char           errorstring[];
                  char           transform_descrip[MAXscreenwidth];
   static         bool           initialized = FALSE;
                  chose_t        Rprec_s,
                                 averprec_s,
                                 interpolate_s,
                                 numdoc_s;
                  chose_t *      level_s[NUMofTYPES];
                  Matrix *       selected_avprec[1];
                  Matrix *       selected_Rprec[1];
                  Matrix *       selected_numdoc[NUMofNUMDOCS];
                  Matrix *       selected_interpol[NUMofINTERPOL];
                  string         parti_name[MAX_PARTICIPANT];
                  string         query_name[NUMofQUERIES];
                  string         level_name[MAX_LEVEL];
                  Matrix **      all_data[NUMofTYPES];
                  Matrix **      chosen[NUMofTYPES];
   register       bool           okay = TRUE;
   register       unsigned short choice;
   register       matrix_dim     type, 
                                 level;
                  bool           display_intermediate_results = FALSE;
                  char           level_list[MAX_LEVEL * (4 + 2) + 1] = "";

   /* initialization */
   if (!initialized) {
      strcpy(transform_descrip, "Raw Data");
      all_data[AvPre]    = &precision_mat[AvPre];
      all_data[RPre]     = &precision_mat[RPre];
      all_data[NumDoc]   = numdoc_mat;
      all_data[Interpol] = interpolate_mat;
      chosen[AvPre]      = selected_avprec;
      chosen[RPre]       = selected_Rprec;
      chosen[NumDoc]     = selected_numdoc;
      chosen[Interpol]   = selected_interpol;
      level_s[AvPre]     = &averprec_s;
      level_s[RPre]      = &Rprec_s;
      level_s[NumDoc]    = &numdoc_s;
      level_s[Interpol]  = &interpolate_s;
      init_name_list(parti_name, MAX_PARTICIPANT);
      init_name_list(query_name, NUMofQUERIES);
      init_name_list(level_name, MAX_LEVEL);
      s_init(&averprec_s,    typename[AvPre],    typesize[AvPre],    \
             typelen[AvPre]);
      s_init(&Rprec_s,       typename[RPre],     typesize[RPre],     \
             typelen[RPre]);
      s_init(&interpolate_s, typename[Interpol], typesize[Interpol], \
             typelen[Interpol]);
      s_init(&numdoc_s,      typename[NumDoc],   typesize[NumDoc],   \
             typelen[NumDoc]);
      init_print_count();

      /* initially: AvPre is selected and nothing else is */
      okay = (s_append(typename[AvPre], &averprec_s, TRUE)) && \
             (s_append(typename[RPre],  &Rprec_s, FALSE));
      for (type=2; type < 4; type++) {
         for (level=0; okay && (level < typesize[type]); level++) {
            okay = s_append(valname[type][level], level_s[type], FALSE);
         }
      }
      if (!okay) {
         swerr(proc,"Not enough memory to initialize status arrays?!",1);
      }
      for (okay=TRUE, type=0; okay && (type < NUMofTYPES); type++) {
         okay = mat_3D_init(chosen[type], typesize[type], NUMofQUERIES, \
                         MAX_PARTICIPANT, typename[type], valname[type]);
         for (level=0; level < typesize[type]; level++) {
            mat_resize(chosen[type][level], 0, 0);
         }
      }
      if (!okay) {
         sprintf(errorstring, "Cannot allocate memory for %s submatrix", \
                              typename[type]);
         swerr(proc, errorstring, 1); /* NOTREACHED */
      }
      initialized = TRUE;
   }


   /* CONSTCOND */
   while (TRUE) {
      start_menu();
      printf("%s -- version %s\n\n", product, version);
      puts("Top level menu");
      puts(" Statistical analyses");
      puts("   1.  Descriptive statistics, tables and graphical plots of "
           "results");
      puts("   2.  Comparison statistics with selected data\n");
      puts(" Select Data to analyse or display");
      puts("   3.  Select (or display) participants and queries");
      puts("   4.  Select measures for analysis or display\n");
      puts(" Miscellaneous options");
      puts("   5.  Data files: reading them and information about them");
      puts("   6.  Change program options"); 
      puts("   7.  About the program\n");
      puts("   8.  Quit\n");   

      choice = getchoice(1,8,PROMPT,0);
      if (0 != choice) {
         switch(choice) {
            case 1: descrip_menu(participant_s, query_s, level_s, \
                                 all_data, chosen, level_list,    \
                                 transform_descrip);
                    break;
            case 2: anova_menu(display_intermediate_results, level_list, \
                               participant_s, query_s, level_s,          \
                               all_data, chosen, transform_descrip);
                    break;
            case 3: data_menu(participant_s, query_s);
                    break;
            case 4: level_menu(level_s, level_list);
                    break;
            case 5: file_menu(precision_mat, interpolate_mat,    \
                              numdoc_mat, participant_s, query_s);
                    break;
            case 6: options_menu(&display_intermediate_results,  \
                                 transform_descrip);
                    break;
            case 7: misc_menu();
                    break;
	    case 8: return(TRUE);
            default: {
                        
	             } 
         } /* switch choice */
      }
   }
   return (okay);
} /* main_menu() */



/*                                                                         *\
   -----------------------------------------------------------------------
			     First Level Menus
   -----------------------------------------------------------------------
\*                                                                         */

/* Menu 0.1 */
void descrip_menu(const chose_t * parti_s,
                  const chose_t * query_s,
                        chose_t * level_s[],
                        Matrix ** data[],
                        Matrix ** chosen[],
                        char      level_list[],
                        string    tran_str)
{ 
          const char           proc[] = "descrip_menu";
   extern const char           PROMPT[];
   extern       char           errorstring[];
                unsigned short choice;
   static       average_type   ave_status = AVER_NONE;
                string         parti_name[MAX_PARTICIPANT];
                string         query_name[NUMofQUERIES];
  register      matrix_dim     type, level, count=0;

   
   /* CONSTCOND */
   while (TRUE) { 
      start_menu();
      printf("%s -- version %s\n\n", product, version);
      puts("Second level menu 1 -- Descriptive Statistics\n");
      puts(" Select output type:");
      puts("   1.  Tables");
      puts("         - precision scores for selected data");
      puts("   2.  Graphs");
      puts("         - histograms for average and R-precision; or");
      puts("         - R-P plots for all interpolated levels\n");
      puts  (" Select level of Averaging");
      printf(" %c 3.  Scores for each participant-query pair (no averaging)\n",
             (AVER_NONE     == ave_status)?on:off);
      printf(" %c 4.  Scores for each participant, averaged over queries\n", \
             (AVER_BY_PARTI == ave_status)?on:off);
      printf(" %c 5.  Scores for each query, averaged over participants\n",  \
             (AVER_BY_QUERY == ave_status)?on:off);
      printf(" %c 6.  Scores averaged over all participants and queries\n\n",\
             (AVER_BY_BOTH == ave_status)?on:off);
      puts("   7.  Return to top level menu\n");   

      choice = getchoice(1,7,PROMPT,7);
      if (0 != choice) {
         if ((1 == choice) || (2 == choice)) {
            matrix_dim partis  = s_count(parti_s);
            matrix_dim queries = s_count(query_s);

            if ((0 != partis) && (0 != queries)) {
               init_name_list(parti_name, MAX_PARTICIPANT);
               init_name_list(query_name, NUMofQUERIES);
	    } else {
               printf("Can't display figures for "MAT_DIM_SPEC" participants "\
                      "and "MAT_DIM_SPEC" queries", partis, queries);
               NewLine(1, stdout);
               end_display();
               break; /* *** BREAK here *** */
	    }
	 }
         switch(choice) {
	   case 1: /* Tables */
              if (NULL == transform(TRANS_BEFORE)) {
                 tran_str = "Raw Data";
	      }
              if (mat_chosen(parti_s, query_s, level_s, data, chosen, \
                  parti_name, query_name, transform(TRANS_BEFORE))) {
                 for (type=0; type < NUMofTYPES; type++) {
                    if (s_count(level_s[type])) {
                       switch (ave_status) {
		          case AVER_NONE:
                             count = s_count(level_s[type]);
                             for (level=0; level < count; level++) {
                                print_raw_data(PARTI_NAME_LEN, QUERY_NAME_LEN,\
                                  chosen[type][level], parti_name, query_name,\
                                  tran_str, (0==level));
			     }
                             break;
		          case AVER_BY_PARTI:
		          case AVER_BY_QUERY:
			  {
                             real averages[MAX_MATRIX];

                             aver_compute((AVER_BY_PARTI == ave_status),   \
                              chosen[type], s_count(level_s[type]), averages);
                             print_ave_data(averages,                      \
                              s_count((AVER_BY_PARTI==ave_status)          \
                                      ?parti_s :query_s),                  \
                              (ave_status)?(ScreenSize)PARTI_NAME_LEN      \
                                          :(ScreenSize)QUERY_NAME_LEN,     \
                              (AVER_BY_PARTI==ave_status)?parti_name       \
                                                         :query_name,      \
                              (AVER_BY_PARTI==ave_status),                 \
                              (level_list[0])?level_list:NULL, tran_str,   \
                              typename[type], (0==level));
			  }
                          break;
		          case AVER_BY_BOTH: 
			     {
                                real both_average=aver_by_both(chosen[type],\
                                                   s_count(level_s[type]));
                                print_both_ave((level_list[0])?level_list:NULL,
                                    tran_str, typename[type], both_average, 
                                    TRUE);
                                break;
			     }
		          default: 
                             sprintf(errorstring, \
                                    "imposssible average type %d", ave_status);
                             swerr(proc, errorstring, 0);
		       } /* switch */
		    } /* if s_count > 0 */
	         } /* for type */
                 end_display();
	      } else {
                 swerr(proc, "can't create matrices of selected data", 0);
	      }
              break;
	   case 2: /* Graphs */
              if (mat_chosen(parti_s, query_s, level_s, data, chosen, \
                           parti_name, query_name, transform(TRANS_BEFORE))) {
                 if (is_3way(level_s)) {
                    int  err; 
                    real *  to_print[NUMofINTERPOL];

                    if (prep_RP_plot(parti_s, query_s, data, chosen, \
                                     parti_name, to_print)) {
                       if (! graph_make(FALSE, typename[Interpol], \
                                        to_print, s_count(parti_s),\
                                        parti_name, "Recall", &err)) {
                          if (user_abort != err) {
                             sprintf(errorstring,                             \
                                   "error #%d detected while preparing graph "\
                                   "file", err);
                             swerr(proc, errorstring, 0);
			  }
                          return;
		       }
		    }
		 } else {
                    real *     to_print[NUMofPRECISION]/*[MAX_HISTO_LABEL]*/;
                    string     label[MAX_HISTO_LABEL];
                    matrix_dim which;
                    matrix_dim num_pts = 0; 
                    string     x_axis  = NULL;
                    char       both;
                    matrix_dim i;

                    for (i = 0; i < NUMofPRECISION; i++) {
                       to_print[i] = (real *)calloc(sizeof(real), \
                                                    MAX_HISTO_LABEL);
		    } 
                    if (NULL == to_print[NUMofPRECISION - 1]) {
                       swerr(proc, "cannot create list of data points", 0);
                       for (i=0; i < NUMofPRECISION; i++) {
                          if (NULL != to_print[i]) {
                             free(to_print[i]);
			  } else {
                             break; 
			  }
		       }
                       return;
		    }

                    switch (ave_status) {
		       case AVER_NONE:
                            num_pts = s_count(parti_s) * s_count(query_s);
                            x_axis = "Participant / Query";
                            /*FALLTHROUGH*/
		       case AVER_BY_PARTI:
                            if (0 == num_pts) {
                               num_pts = s_count(parti_s);
                               x_axis = "Participant";
			    }
                            /*FALLTHROUGH*/
		       case AVER_BY_QUERY: {
                            if (0 == num_pts) {
                               num_pts = s_count(query_s);
                               x_axis = "Query";
			    }

                          both = prep_histo(parti_name, query_name, level_s, \
                                     ave_status, chosen, label, to_print,    \
                                     &which);
                          if ((1 == both) || (2 == both)) {
                             int err = both;

                             if (! graph_make(TRUE, x_axis, to_print, num_pts,\
                                 label, s_title(level_s[which]), &err)) {
                                if (user_abort != err) {
                                   sprintf(errorstring, \
                                          "error #%d reported making graph " \
                                          "file", err);
                                   swerr(proc, errorstring, 0);
				}
			     }
			  } else {
                             swerr(proc, "serious error from prep_histo()", 1);
                             /*NOTREACHED*/
			  }
		       }
                       break;
		       case AVER_BY_BOTH: 
                          fputs("Cannot create a histogram with only " \
                                "one datum", stdout);
                          NewLine(1, stdout);
                       break;
		       default: 
                           sprintf(errorstring,                   \
                                   "imposssible average type %u", \
                                   ave_status);
                           swerr(proc, errorstring, 0);
		    } /* switch ave_status if !3way-data in plot */
		 }
                 end_display();
	      } else {
                 swerr(proc, "can't create matrices of selected data", 0);
	      }
              break;
	   case 3:
              ave_status = AVER_NONE;
              break;
	   case 4:
              ave_status = AVER_BY_PARTI;
              break;
	   case 5:
              ave_status = AVER_BY_QUERY;
              break;
	   case 6:
              ave_status = AVER_BY_BOTH;
              break;
	   case 7:
              return;
	   default:
              swerr(proc, "impossible choice", 1);
	 } /* switch */
      }
   } /* while TRUE */
} /* descrip_menu() */


/* Menu 0.2 */
void anova_menu(const bool      inter_results,
                const string    level_list,
                const chose_t * parti_s,
                const chose_t * query_s,
                      chose_t * level_s[],
                      Matrix ** data[],
                      Matrix ** chosen[],
                      char      trans_str[])
{
       const char           proc[] = "anova_menu";
extern const char           PROMPT[];
             unsigned short choice = 0;
             string         parti_name[MAX_PARTICIPANT];
             string         query_name[NUMofQUERIES];
             matrix_dim     type, level, count;
             bool           simple_anova;

   if ((2 > s_count(parti_s)) || (2 > s_count(query_s))) {
      start_menu();
      fputs("Cannot compute an ANOVA without at least two participants" , \
            stdout);
      NewLine(1, stdout);
      fputs("and two queries.", stdout);
      NewLine(1, stdout);
      end_display();
      return;
   }
   init_name_list(parti_name, MAX_PARTICIPANT);
   init_name_list(query_name, NUMofQUERIES);
   init_print_count();
   for (type=0; type < NUMofTYPES; type++) {
      if (0 != (count = (s_count(level_s[type])))) {
         if (mat_chosen(parti_s, query_s, level_s, data, chosen, parti_name, \
                        query_name, transform(TRANS_BEFORE))) {
            simple_anova = (1 == s_count(level_s[type]));
            if (! simple_anova) {
               while (0 == choice) { 
                  start_menu();
                  printf("%s -- version %s\n\n", product, version);
                  puts("Second level menu 2 -- Comparison Statistics " \
                       "(3-way data)\n");
                  printf("ANOVA Test for a Difference in Mean Scores:\n" \
                        "       %s\n",typename[type]);

                  puts("  1.  Two-factor design   " \
                              "(a two-way ANOVA for each level)");
                  puts("  2.  Three-factor design (all levels at once)");
                  puts("  3.  Return to top level menu\n");

                  choice = getchoice(1,3,PROMPT,3);
                  if (0 != choice) {
                     if (3 == choice) {
                        return;
	             } 
                     if ((1 == choice) || (2 == choice)) {
                        simple_anova = (1 == choice);
	             } else {
                        swerr(proc, "impossible choice", 1);
	             }
                  }
               } /* while */
	    } /* if ! simple_anova */
         } else {
            swerr(proc, \
                  "Can't make submatrices for ANOVAs (this is very bad)", 0);
         }
         if (simple_anova) {
            for (level=0; level < count; level++) {
               TwoWayStats(chosen[type][level], typename[type], parti_name,   \
                        query_name, (1 != typesize[type])                     \
                                       ?mat_level(chosen[type][level]) :NULL, \
                        trans_str, inter_results);
            }
	 } else {
            ThreeWayStats(chosen[type], count, typename[type], level_list, \
                          parti_name, query_name, trans_str, inter_results);
	 }
      }
   }
} /* anova_menu() */


/* Menu 0.3 */
void data_menu(chose_t * parti_s,
               chose_t * queri_s) 
{
                char           proc[] = "data_menu";
                unsigned short choice;
                elem_t         printed[MAX_MATRIX]; 
                bool           err;
   extern const char           PROMPT[];

   start_menu();
   /* CONSTCOND */
   while (TRUE) { 
      start_menu();
      printf("%s -- version %s\n\n", product, version);
      puts("Second Level Menu 3 -- Participants and Queries\n");
      puts("     Display");
      puts("        1.  selected data           2.  all data\n");
      puts("     Insert");
      puts("        3.  one participant         4.  all participants");
      puts("        5.  one query               6.  all queries\n");
      puts("     Remove");
      puts("        7.  one participant         8.  all participants");
      puts("        9.  one query              10.  all queries\n");
      puts("     11.  Return to previous menu\n\n");
      printf("   >> "MAT_DIM_SPEC" participant%s and "MAT_DIM_SPEC" quer%s " \
             "selected <<\n\n",               \
             s_count(parti_s),                \
             (1==s_count(parti_s))?"":"s",    \
             s_count(queri_s),                \
             (1==s_count(queri_s))?"y":"ies");

      choice = getchoice(1,11,PROMPT,11);
      if (0 != choice) {
         switch(choice) {
            case 1: {
                       fputs("-- Selected participants and queries --",stdout);
                       NewLine(1, stdout);
                       init_print_count();
                       if (0 != s_count(parti_s)) {
                          fputs(" - Selected participants -", stdout);
                          NewLine(1, stdout);
                          s_print(parti_s, FALSE, TRUE, printed, \
                                  width_display());
		       } else {
                          fputs("No participants are selected.", stdout);
                          NewLine(1, stdout);
		       }
                       if (0 != s_count(queri_s)) {
                          fputs(" - Selected queries -", stdout);
                          NewLine(1, stdout);
                          s_print(queri_s, FALSE, TRUE, printed, \
                                  width_display());
		       } else {
                          fputs("No queries are selected.", stdout);
                          NewLine(1, stdout);
		       }
                       end_display();
	            }
                    break;
            case 2: 
                    init_print_count();
                    fputs("-- All participants and queries --", stdout);
                    NewLine(1, stdout);
                    fputs(" - All participants -", stdout);
                    NewLine(1, stdout);
                    s_print(parti_s, TRUE, TRUE, printed, width_display());
                    fputs(" - All queries -", stdout);
                    NewLine(1, stdout);
                    s_print(queri_s, TRUE, TRUE, printed, width_display());
                    end_display();
                    break;
            case 3: /* insert 1 participant */
                    init_print_count();
                    if (s_all_are_on(parti_s)) {
                       fputs("All particpants are already selected.",
                             stdout);
                       NewLine(1, stdout);
		    } else {
                       unsigned short change;
                       elem_t         num_printed;

                       init_print_count();
                       num_printed=s_print(parti_s, FALSE, FALSE, printed, \
                               width_display());
                       change=getchoice(0,num_printed,                     \
                                  "Enter number of participant to insert " \
                                  "(or 0 to quit) ", 0);
                       if (0 != change) {
                          s_toggle(parti_s, printed[change - 1]);
                          printf("Inserted participant #%hu (%s).\n", \
                                 change, s_name(parti_s,              \
                                                printed[change -1], &err));
                       } 
		    }
                    end_menu();
                    break;
            case 4: /* insert all participants */
                    s_all_select(parti_s, TRUE);         
                    puts("Inserted all participants.");
                    end_menu();
                    break;
            case 5: /* insert 1 query */
                    init_print_count();
                    if (s_all_are_on(queri_s)) {
                       puts("All queries are already inserted.");
                    } else {
                       unsigned short change;
                       elem_t         num_printed;

                       num_printed=s_print(queri_s, FALSE, FALSE, printed, \
                               width_display());
                       change=getchoice(0, num_printed,                 \
                                     "Enter number of query to insert " \
                                     "(or 0 to quit) ", 0);
                       if (0 != change) {
                          s_toggle(queri_s, printed[change - 1]);
                          printf("Inserted query #%hu (%s)\n",            \
                                 change, s_name(queri_s,                  \
                                                printed[change -1], &err));
                       } 
                    }
                    end_menu();
                    break;
            case 6: /* insert all queries */
                    s_all_select(queri_s, TRUE);
                    puts("Inserted all queries.");
                    end_menu();
                    break;
            case 7: { /* Remove one participant */
                          unsigned short change;
                          elem_t         num_printed;

                       if (0 == s_count(parti_s)) {
                          puts("No participants are selected, so you can't " \
                               "remove any.");
		       } else {
                          init_print_count();
                          num_printed=s_print(parti_s, FALSE, TRUE, printed, \
                                  width_display());
                          change=getchoice(0, num_printed,
                                     "Enter number of participant to insert " \
                                     "(or 0 to quit) ", 0);
                          if (0 != change) {
                             s_toggle(parti_s, printed[change - 1]);
                             printf("Removed participant #%hu (%s).\n", \
                                    change, s_name(parti_s,             \
                                                   printed[change -1], &err));
                          } 
		       }
                    }
                    end_menu();
                    break;
            case 8: /* Remove all participants */
                    s_all_select(parti_s, FALSE);
                    puts("Removed all particpants");
                    end_menu();
                    break;
            case 9: /* Remove one query */
                    if (0 == s_count(queri_s)) {
                       puts("No queries are selected, so you can't " \
                            "remove any.");
                       end_menu();
		    } else {
                       unsigned short change;
                       elem_t         numprinted;

                       init_print_count();
                       numprinted=s_print(queri_s, FALSE, TRUE, printed, \
                               width_display());
                       change=getchoice(0, numprinted,               \
                                  "Enter number of query to insert " \
                                  "(or 0 to quit) ", 0);
                       if (0 != change) {
                          s_toggle(queri_s, printed[change - 1]);
                          printf("Removed query #%hu (%s).\n", \
                                 change, s_name(queri_s,       \
                                                printed[change -1], &err));
                       } 
		    }
                    break;
            case 10: /* remove all queries */
                     s_all_select(queri_s, FALSE);
                     puts("Removed all queries");
                     end_display();
                    break;
            case 11: /* Return to previous menu */
                     return; /* NOTREACHED */
                    break;
            default: { 
               sprintf(errorstring, "impossible case %hu\n", choice);
               swerr(proc, errorstring, 1); /* NOTREACHED */
   	    }
         } /* switch */
      } else {
         sprintf(errorstring, "impossible case %hu\n", choice);
         swerr(proc, errorstring, 1); /* NOTREACHED */
      }
    } /* while */
} /* data_menu() */


/* Menu 0.4 */
void level_menu(chose_t * level_select[],
                char      level_list[])
{
          const char           proc[] = "level_menu";
                unsigned short choice;
   extern const string *       valname[];
   extern const char           PROMPT[];

   /* CONSTCOND */
   while (TRUE) { 
      start_menu();
      printf("%s -- version %s\n\n", product, version);
      puts("Second level menu 4 -- Measures\n");
      puts  (" Select measures for 2- or 3-way analysis, or display:");
      printf ("   1.  Precision at fixed recall level%s", \
             ((0 != s_count(level_select[Interpol])) &&   \
             (NULL != strchr(level_list, ',')))?"s":"");
         if ((0 != s_count(level_select[Interpol])) && (level_list[0])) {
            printf(":\n        %s\n", level_list);
         } else {
            fputs("\n", stdout);
	 }
      fputs("   2.  Precision at fixed number of retrieved documents",stdout);
         if ((0 != s_count(level_select[NumDoc])) && (level_list[0])) {
            printf(":\n        %s\n\n", level_list);
         } else {
            fputs("\n\n", stdout);
	 }
      puts  (" Select measures for 2-way analysis, or display:");
      printf(" %c 3.  Average precision\n", \
             (0 != s_count(level_select[AvPre]))?on:off);
      printf(" %c 4.  R-precision\n\n",     \
             (0 != s_count(level_select[RPre]))?on:off);
      puts  ("   5.  Return to second level menu\n\n");   

      choice = getchoice(1,5,PROMPT,5);
      if (0 != choice) {
         switch(choice) {
	    case 1:
              if (s_select_menu(level_select[Interpol])) {
                 make_level_list(level_list, level_select[Interpol], \
                                 valname[Interpol]);
                 s_all_select(level_select[NumDoc], FALSE);
                 s_all_select(level_select[AvPre], FALSE);
                 s_all_select(level_select[RPre],  FALSE);
	       }
               break;
	    case 2: 
              if (s_select_menu(level_select[NumDoc])) {
                 make_level_list(level_list, level_select[NumDoc], \
                               valname[NumDoc]);
                 s_all_select(level_select[Interpol], FALSE);
                 s_all_select(level_select[AvPre], FALSE);
                 s_all_select(level_select[RPre],  FALSE);
	       }
               break;
	    case 3:
               s_toggle(level_select[AvPre], 0);
               if (s_value(level_select[AvPre], 0)) {
                 s_all_select(level_select[Interpol], FALSE);
                 s_all_select(level_select[NumDoc],   FALSE);
                 level_list[0] = '\0';
	       }
               break;
	    case 4:
               s_toggle(level_select[RPre], 0);
               if (s_value(level_select[RPre], 0)) {
                 s_all_select(level_select[Interpol], FALSE);
                 s_all_select(level_select[NumDoc],   FALSE);
                 level_list[0] = '\0';
	       }
               break;
	    case 5:
               return;
	    default:
              swerr(proc, "impossible choice", 1);
	 } /* switch */
      }
   } /* while TRUE */
} /* level_menu() */


/* Menu 0.5 */
void file_menu(Matrix *  prec[],
               Matrix *  interp[],
               Matrix *  numdoc[],
               chose_t * parti_s,
               chose_t * query_s)
{
            const char  proc[] = "file_menu";
   extern   const char  PROMPT[];
                  short choice;
                  char  filename[MAXFNAMELEN];

   /* CONSTCOND */
   while (TRUE) { 
      start_menu();
      printf("%s -- version %s\n\n", product, version);
      puts("Second level menu 5 -- Data files\n");
      puts(" Select option:");
      puts("    1.  Read in a datafile to supplement current data\n");
      puts("   Datafile Format");
      puts("    2.  Describe format       3.  Display example\n");
      puts("   Display scripts used to create data files");
      puts("    4.  extract.sh            5.  extract.awk\n");
      puts("    6.  Return to top level menu\n\n");

      choice = getchoice(1,6,PROMPT,6);
      if (0 != choice) {
         switch(choice) {
	    case 1: GetStringFromUser(filename, MAXFNAMELEN - 1U,            \
                         "Please enter filename (or press Enter to cancel) ");
                    if ('\n' != filename[0]) {
                       chop(filename);
                       if (0 != read_data(filename, prec, interp, numdoc, \
                                          parti_s, query_s)) {
                          return; /* NOTREACHED */
                       }
                    }
                    break;
	    case 2: display_file(Format);/*file_format();*/
                    break;
	    case 3: display_file(Example);/*format_example();*/
                    break; 
	    case 4: display_file(Sh);/*shell_script();*/
                    break;
	    case 5: display_file(Awk);/*awk_script();*/
                    break;
            case 6: return; /* NOTREACHED */
                    break;
            default: { 
               sprintf(errorstring, "impossible case %d\n", choice);
               swerr(proc, errorstring, 1);
               /* NOTREACHED */
   	    }
         } /* switch */
      } else {
         swerr(proc, "error from getchoice()", 1); 
         /* NOTREACHED */
      }
    } /* while */
} /* file_menu() */


/* Menu 0.6 */
void options_menu(bool * inter_results,
                  char   transform_str[])
{
            const char           proc[] = "options_menu";
   extern   const char           PROMPT[];
   extern         char           errorstring[];
   register       ScreenSize     height = height_display(); 
   register       ScreenSize     width  = width_display();
                  unsigned short choice;

   /* CONSTCOND */
   while (TRUE) { 
      start_menu();
      printf("%s -- version %s\n\n", product, version);
      puts("Second level menu 6 -- Program Options\n");
      puts(" Display options");
      printf("   1.  Show intermediate results during computation: %s\n", \
             (*inter_results)?"yes":"no");
      printf("   2.  Clear display before presenting menus: %s\n", \
             (s_m_value())?"yes":"no");
      printf("   3.  Maximum width of tables: "SSZ_SPEC" columns\n", width);
      if (0 == height) {
         printf("   4.  Height of display:       infinite lines\n");
      } else {
         printf("   4.  Height of display:       "SSZ_SPEC" lines\n", height);
      }
      

      puts("\n Transform Data");
      printf("   5.  When to use transform:  %s\n", when_descrip());
      printf("   6.  Which transform to use: ");
      if (! never_transform()) {
         printf("%s transform", tran_name());
      } else {
         printf("[%s transform]", tran_name());
      }
      puts("\n");
      
      puts("   7.  Return to top level menu\n");   

      choice = getchoice(1,7,PROMPT,7);
      if (0 != choice) {
         switch(choice) {
	    case  1: 
                    (*inter_results) = (*inter_results)?FALSE:TRUE;
                    break;
	    case  2: s_m_toggle();
                     break;
            case  3: 
                    width = getchoice(1, MAXscreenwidth,           \
                                      "Enter new display width: ", \
                                      width_display());
                    if (0 != choice) {
                       set_width(width);
		    }
                    break;
            case  4: 
                     height = (ScreenSize)getchoice(0, MAXscreenlines,       \
                                    "Enter new display length (0 for oo): ", \
                                    height_display());
                     if (0 != choice) {
                        set_height(height);
		     }
                     break;
	    case  5: /* when to use transform */
                     when_trans_menu(product, version, \
                                     "Third level menu 6.4\n");
                     break;
	    case  6: /* choose transform */
                     tran_choose(product, version, \
                                 "Third level menu 6.5\n");
                     break;
            case  7: 
                    if (never_transform()) {
                       strcpy(transform_str, "Raw data");
		    } else {
                       sprintf(transform_str, "After %s transform applied",\
                                              tran_name());
                    }   
                    return; 
            default: { 
               sprintf(errorstring, "impossible case %hu", choice);
               swerr(proc, errorstring, 1);
               /* NOTREACHED */
   	    }
         } /* switch */
      } else {
         swerr(proc, "error from getchoice()", 1);
         /* NOTREACHED */
      }
    } /* while */
} /* options_menu() */



/* Menu 0.7 */
void misc_menu(void)
{
            const char           proc[] = "misc_menu";
   extern   const char           PROMPT[];
   extern         char           errorstring[];
                  unsigned short choice;

   /* CONSTCOND */
   while (TRUE) { 
      start_menu();
      printf("%s -- version %s\n\n", product, version);
      puts("Second level menu 7 -- About the program\n");

      puts(" Select information to display");
      puts("   1.  Program overview");
      puts("   2.  Display program limits");
      puts("   3.  Display reference list");
      puts("   4.  Authors' names and addresses");
      puts("   5.  Display disclaimers for software used here\n");
      puts("   6.  Return to top level menu");
      choice = getchoice(1,6,PROMPT,6);
      if (0 != choice) {
         switch(choice) {
	    case  1: display_file(Overview);
                     break;
            case  2: limits();
                     break;
            case  3: display_file(Refer);
                     break;
            case  4: display_file(Author);
                     break;
            case  5: display_file(f2c);
                     break;
            case  6: return;
            default: { 
               sprintf(errorstring, "impossible case %hu", choice);
               swerr(proc, errorstring, 1);
               /* NOTREACHED */
   	    }
         } /* switch */
      } else {
         swerr(proc, "error from getchoice()", 1);
         /* NOTREACHED */
      }
    } /* while */
} /* misc_menu() */


/*                                                                         *\
   -----------------------------------------------------------------------
                        Utility Routines for Menus
   -----------------------------------------------------------------------
\*                                                                         */

bool is_3way(chose_t * level_s[])
{
   const    char           proc[] = "is_3way";
   register unsigned short type;

   for (type=0; type < NUMofTYPES; type++) {
      if (0 < s_count(level_s[type])) {
         return ((1 < s_size(level_s[type])));
      }
   }
   swerr(proc, "nothing is selected", 0);
   return(FALSE);
} /* is_3way() */


bool prep_RP_plot(const chose_t * parti_s,
                  const chose_t * query_s,
                        Matrix ** all_data[],
                        Matrix ** interp_only[],
                        string    parti_name[],
                        real *    average[NUMofINTERPOL]) {

          const char      proc[] = "prep_RP_plot";
   extern const elem_t    typelen[];
   extern const string    typename[];
   extern const elem_t    typesize[];
   extern const string *  valname[];
                chose_t   fixed_prec;
                chose_t   nothing_selected;
                chose_t * level_s[NUMofTYPES];
                bool      okay = TRUE;
                elem_t    level;

   s_init(&nothing_selected, "-Nothing-", 0, 0);
   level_s[AvPre] = level_s[RPre] = level_s[NumDoc] = &nothing_selected;
   level_s[Interpol] = &fixed_prec;
   init_name_list(parti_name, s_size(parti_s));
   for (okay=TRUE,level=0; okay && (level < NUMofINTERPOL); level++) {
      okay = (NULL != (average[level] = (real *)calloc(MAX_PARTICIPANT, \
                                                       sizeof(real))));
   }
   if (! okay) {
      elem_t remove_me;

      swerr(proc, "can't collect interpolated data", 0);
      for (remove_me = 0; remove_me < level; remove_me++) {
         free(average[remove_me]);
      }
      return(FALSE);
   }

   if (TRUE == (okay = s_init(&fixed_prec, typename[Interpol],         \
                              typesize[Interpol], typelen[Interpol]))) {
      for (level=0; okay && (level < typesize[Interpol]); level++) {
         okay = s_append(valname[Interpol][level], &fixed_prec, TRUE);
      }
   } 
   if (! okay) {
      swerr(proc, "cannot make selection vector -- cannot make R-P plot", 0);
      return(FALSE);
   }

   for (level=0; level < typesize[Interpol]; level++) {
      s_all_select(&fixed_prec, FALSE);   /* clear all fixed_prec values */
      s_assign(&fixed_prec, level, TRUE); /* set pos level in fixed_prec */

      if (! mat_chosen(parti_s, query_s, level_s, all_data, interp_only, \
                       parti_name, NULL, transform(TRANS_BEFORE))) {
         swerr(proc, "error making copy of selected data", 0);
         return(FALSE);
      } else {
         aver_compute(TRUE, interp_only[Interpol], 1, average[level]);
      }
   }
   return(TRUE);
} /* prep_RP_plot() */



char prep_histo(const string       parti_name[],
                const string       query_name[],
                      chose_t *    level_s[],
                const average_type ave_status,
                      Matrix **    chosen[],
                      string       label[MAX_HISTO_LABEL],
                      real *       toprint[NUMofPRECISION],
                      matrix_dim * which) 
{
/*
   returns: 0 -> error
            1 -> AvPre or RPre in toprint[0]
            2 -> AvPre in toprint[0], RPre in toprint[1]
*/
   #define MAX_LABEL_LEN PARTI_NAME_LEN + 3 + QUERY_NAME_LEN + 1
   const    char       proc[] = "prep_histo";
            matrix_dim max_parti = 0; 
            matrix_dim max_query = 0; 
   const    matrix_dim prec_type[NUMofPRECISION] = {AvPre, RPre};
   register matrix_dim parti;
   register matrix_dim query;
   register matrix_dim prec;
            matrix_dim count = 0;
   #undef MAX_LABEL_LEN

   init_name_list(label, MAX_HISTO_LABEL);
   for (prec=0; (0 == max_parti) && (prec < NUMofPRECISION); prec++) {
      max_parti = num_cols(*chosen[prec]);
      max_query = num_rows(*chosen[prec]);
   }
   
   if (histo_labels(label, parti_name, query_name, max_parti, max_query, \
                    ave_status)) {
      for (prec=0; prec < NUMofPRECISION; prec++) {
         if (s_count(level_s[prec_type[prec]])) {
            register matrix_dim i;

            *which = prec;
            if (AVER_NONE == ave_status) {
               for (parti=i=0; parti < max_parti; parti++) {
                  for (query=0; query < max_query; query++, i++) {
                     toprint[count][i]=mat_value(chosen[prec_type[prec]][0], \
                                                 query, parti);
		  }
	       }
	    } else {
               aver_compute((AVER_BY_PARTI == ave_status),     \
                            chosen[prec_type[prec]], 1,       \
                            toprint[count]);
	    }
            count++;
	 } /* if s_count() */
      }
   } else {
      swerr(proc, "cannot make a histogram without labels", 0);
   }
   return(count);
} /* prep_histo() */


bool histo_labels(      string       label[],
                  const string       parti_name[],
                  const string       query_name[],
                  const matrix_dim   max_parti,
                  const matrix_dim   max_query,
                  const average_type ave_status)
{
   const char proc[] = "histo_labels";
   register matrix_dim parti;
   register matrix_dim query;
   static   bool       initialized = FALSE;

   if (! initialized) {
      init_name_list(label, MAX_HISTO_LABEL);
      initialized = TRUE;
   }

   switch (ave_status) {
      case AVER_NONE: {
         register unsigned long i;

         for (parti=i=0; parti<max_parti; parti++) {
            for(query=0; query<max_query; i++,query++) {
               label[i]=(string)malloc(strlen(parti_name[parti]) + 3 + \
                                       strlen(query_name[query]) + 1);
               if (NULL == label[i]) {
                  swerr(proc, "Not enough memory for labels", 0);
                  return(FALSE);
	       } else {
                  sprintf(label[i], "%s / %s", \
                          parti_name[parti], query_name[query]);
	       }
	    }
         }
      }
      return(TRUE); /*NOTREACHED*/
      break;
      case AVER_BY_PARTI:
         for (parti=0; parti < max_parti; parti++) {
            label[parti] = strdup(parti_name[parti]);
            if (NULL == label[parti]) {
               swerr(proc, "error making list of participant names", 0);
               return (FALSE);
            }
	 }
      break;
      case AVER_BY_QUERY:
         for (query=0; query < max_query; query++) {
            label[query] = strdup(query_name[query]);
            if (NULL == label[query]) {
               swerr(proc, "error making list of query names", 0);
               return (FALSE);
            }
	 }
      break;
      default:
         swerr(proc,"impossible to make labels for this type of histogram",0);
         return(FALSE);
   } /* switch */
   return(TRUE);
} /* histo_labels() */

