/* read.c */

#include <stdio.h>
#include <errno.h>
#include <malloc.h>
#include "types.h"
#include "proto.h"
#include "select.h"
#include "matrix.h"
#include "read.h"

#define MAXWORDLEN (1024)

static bool read_query(FILE * file, chose_t * query_s, unsigned long * linenum,
      Matrix * precision[], Matrix * interpolated[], Matrix * numdoc[], 
      const matrix_dim which_query, const matrix_dim which_parti, 
      const string filename, const bool use_name);
static bool ReadLine (FILE * infile, unsigned long * linenum, 
      string input_buffer, int maxlen, const string filename, 
      const char proc[]);
static bool ReadStringFromFile (FILE * file, int maxlen, string dest, 
      unsigned long * linenum, const string value_type, 
      const string filename, const char proc[]);

/*                                                                         *\
   -----------------------------------------------------------------------
                               Read Datafile
   -----------------------------------------------------------------------
\*                                                                         */

unsigned long read_data(const string      filename,
                              Matrix *    precision[],
                              Matrix *    interpolated[],
                              Matrix *    numdoc[],
                              chose_t *   Parti,
                              chose_t *   Queri) 
{
                     char       proc[] = "read_data";
   extern            char       errorstring[];
   register          matrix_dim qnum;
   register          matrix_dim count=(matrix_dim)0;/* total partis this file*/
                     matrix_dim total=s_count(Parti); 
                     bool       first_time;
                     FILE *     mfile;
                     bool       is_okay = TRUE;
                     char       name[PARTI_NAME_LEN];
            unsigned long       linenum = 0UL;
   const    unsigned long       error = 0UL; /* what to return if 
                                                  there is an error */

   first_time = (0 == total);
   if ( (mfile=open_file(proc,filename,(const string)"r")) != NULL) {
      linenum=1;
      printf("Reading datafile `%s', please wait...\n", filename);
      while(!feof(mfile)) {
         /* get participant */
         if (ReadStringFromFile(mfile,    PARTI_NAME_LEN,     name,
                                &linenum, "participant name", filename, proc)){
            if (feof(mfile)) { 
               printf("   read %lu lines comprising "MAT_DIM_SPEC" complete " \
                      "participants\n",                                       \
                      (linenum - 1), count);
               return(linenum - 1);
            } 
            chop(name);
            if (s_append(name, Parti, TRUE)) {
               for (qnum=0; (qnum < NUMofQUERIES) && (is_okay); qnum++) {
                  is_okay=read_query(mfile, Queri, &linenum,           \
                                     precision, interpolated, numdoc,  \
                                     qnum, total+count, filename, first_time);
               }
               if (! is_okay) { /* read_query() error, already reported */
                  if (! s_remove(Parti)) {
                     swerr(proc, "error removing incomplete participant", 0);
		  }
                  printf("   read %lu lines comprising "MAT_DIM_SPEC  \
                         " complete participants\n"                   \
                         "   skipping the rest of the file\n",        \
                         (linenum - 1), count);
                  close_file(proc, filename, mfile);
                  return (error);
               } else {
                  first_time = FALSE;
                  count++;
               }
	    } else {
               sprintf(errorstring, 
                       "can't have anymore participants\n   -- after " \
                       "reading %lu lines comprising "MAT_DIM_SPEC" "  \
                       "complete participants",                        \
                       (linenum - 1), count);
                swerr(proc, errorstring, 0);
                return(close_file(proc, filename, mfile))?(linenum - 1):error;
	    }
         } else { /* err reading string, reported by ReadStringFromFile() */
            printf("   read %lu lines comprising "MAT_DIM_SPEC" complete " \
                   "participants\n"                                        \
                   "   skipping the rest of the file\n",                   \
                   (linenum - 1), count);
            close_file(proc,filename,mfile);
            return (error);
         }
      } /* while !feof() */
      return(close_file(proc,filename,mfile)?(linenum - 1):(error));
   } else { /* error opening file, reported by open_file() */
      return (error);
   }
} /* read_data() */


bool read_query(      FILE  *         file,
                      chose_t *       query_s,
                      unsigned long * linenum,
                      Matrix *        precision[],
                      Matrix *        interpolated[],
                      Matrix *        numdoc[],
                const matrix_dim      which_query,
                const matrix_dim      which_parti,
                const string          filename,
                const bool            use_name)
{

#define ReadFileError(action) {                             \
   extern char errorstring[];                               \
                                                            \
   sprintf(errorstring, "file %s\n   error %s at line %lu", \
                        filename, action, (*linenum - 1));  \
   swerr(proc, errorstring, 0);                             \
   return(FALSE);                                           \
}

   extern int  errno;
   const  char proc[] = "read_query";
          char inbuff[MAXWORDLEN];
          int  result;

   /* line 2 : query name */
   if (! ReadLine(file,linenum,inbuff,MAXWORDLEN,filename,proc)) {
      ReadFileError("reading line");
   } else {
      if (use_name) {
         chop(inbuff);
         if (! s_append(inbuff, query_s, TRUE)) {
            ReadFileError("inserting query name");
         }
      }
   }

   /* line 3 : interpolated recall at fixed precision */
   if (! ReadLine(file,linenum,inbuff,MAXWORDLEN,filename,proc)) {
      ReadFileError("reading line of interpolated recall values");
   } else {
      real inter_prec[NUMofINTERPOL];
      /*
         The array should be declared of type long double and read
         as such (%Lf).  However, the values are read incorrectly 
         and attempts to print them out cause core dumps.
      */

      chop(inbuff);
      errno = 0;
      result=sscanf(inbuff, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", \
                    &(inter_prec[0]), &(inter_prec[1]), &(inter_prec[2]),  \
                    &(inter_prec[3]), &(inter_prec[4]), &(inter_prec[5]),  \
                    &(inter_prec[6]), &(inter_prec[7]), &(inter_prec[8]),  \
                    &(inter_prec[9]), &(inter_prec[10]));
      if (NUMofINTERPOL == result) { 
         register unsigned int level;
         register bool     okay = TRUE;

         for (level=0; okay && (level < NUMofINTERPOL); level++) {
            okay = (mat_set(interpolated[level],      \
                            which_query, which_parti, \
                            (real)inter_prec[level]));
	 }
         if (!okay) {
            ReadFileError("inserting interpolated recall values");
	 }
      } else { /* error detected during sscanf() */
        #ifdef DEBUG
            swerr(proc,"error scanning interpolated recall values",0);
            fprintf(stderr, "DEBUG trace: inbuff = `%s' result=", inbuff);
            if (EOF == result) {
               fprintf(stderr, "EOF\n");
 	   } else {
               fprintf(stderr, "%d\n", result);
           }
        #endif
        ReadFileError("scanning interpolated recall values");
      }
   }

   /* line 4 : average precision */
   if (! ReadLine(file,linenum,inbuff,MAXWORDLEN,filename,proc)) {
      ReadFileError("reading line with average precision");
   } else {
      real precision_value;

      if (1 == sscanf(inbuff, "%lf ", &(precision_value))) {
         mat_set(precision[AvPre], which_query, which_parti, precision_value);
      } else {
         ReadFileError("scanning average precision");
      }
   }
   
   /* line 5 : precision at fixed document retrieval levels */
   if (! ReadLine(file,linenum,inbuff,MAXWORDLEN,filename,proc)) {
      ReadFileError("reading line with precisions at fixed doc retr'l levels");
   } else {
      real num_prec[NUMofNUMDOCS];

      errno = 0;
      result=sscanf(inbuff, "%lf %lf %lf %lf %lf %lf %lf %lf %lf ", \
                    &(num_prec[0]), &(num_prec[1]), &(num_prec[2]), \
                    &(num_prec[3]), &(num_prec[4]), &(num_prec[5]), \
                    &(num_prec[6]), &(num_prec[7]), &(num_prec[8]));
      if (NUMofNUMDOCS == result) { 
         register unsigned int level;
         register bool         okay = TRUE;

         for (level=0; okay && (level < NUMofNUMDOCS); level++) {
            okay = (mat_set(numdoc[level], which_query, which_parti, \
                            num_prec[level]));
	 }
         if (!okay) {
            ReadFileError("inserting NumDoc values");
	 }
      } else { /* error detected during sscanf() */
         ReadFileError("error scanning precision at fixed number of " \
                       "documents retrieved");
      }
   }   

   /* line 6 : precision after num_rel documents retrieved */
   if (! ReadLine(file,linenum,inbuff,MAXWORDLEN,filename,proc)) {
      ReadFileError("reading line with R-precision");
   } else {
      real precision_value;

      if (1 == sscanf(inbuff, "%lf ", &(precision_value))) {
         mat_set(precision[RPre], which_query, which_parti, precision_value);
      } else {
         ReadFileError("inserting R-precision");
      }
   }

   return (TRUE);
} /* read_query() */


/*                                                                         *\
   -----------------------------------------------------------------------
                           Stream Input Routines
   -----------------------------------------------------------------------
\*                                                                         */

bool ReadLine (     FILE *          infile,
                    unsigned long * linenum,
                    string          input_buffer,
                    int             maxlen,
              const string          filename,      
              const char            proc[]) 
{
   extern char errorstring[];

   if (fgets(input_buffer, (int)maxlen, infile) != NULL) {
      (*linenum)++;                                                          
      return(TRUE);
   } else {                                                      
      int ErrNo = errno; /* save errno so feof() doesn't overwrite it */

      if (!feof(infile)) {
         errno = ErrNo;
         sprintf(errorstring, "read error on file `%s' at line %lu\n", \
                               filename, *linenum);
         swerr(proc, errorstring, 0);
         return(FALSE);
      } else {
         return(TRUE);
      }
   }
} /* ReadLine() */


bool ReadStringFromFile(      FILE *          file, 
                              int             maxlen,
                              string          dest,
                              unsigned long * linenum,
                        const string          value_type,
                        const string          filename,
                        const char            proc[]) {

   extern char errorstring[];

   if (ReadLine(file,linenum,dest,maxlen,filename,proc)){
      if (strlen(dest) > maxlen) { 
         sprintf(errorstring, "file %s line %lu warning:\n%s is too long.\n" \
                              "I'm only using the first %d characters.",     \
                              filename, *linenum, value_type, maxlen);
         swerr(proc, errorstring, 0);
         dest[maxlen - 1]='\0';
      } 
      return TRUE;
   } else { 
      /* ReadLine() has reported the error */
      return FALSE;
   } 
} /* ReadStringFromFile() */


