#!/bin/awk

#
# title
# ylabel
# number of types of bar charts (1=Average precision or R-precision, 2=both)
# number of data points
# foreach curve:
#   curve name
#   datapoints
#

BEGIN {
   patt[0] = "fill 1.0 pattern stripe  40";
   patt[1] = "fill 0.9 pattern stripe -40";
   gap = 0.5   # space between bars for different samples
   width = 1.0 # how wide bars should be

   getline title
   getline ylabel
   getline; numcurves=$1; numpts=$2
      if ((numcurves > 2) || (numcurves < 1)) {
         print "Error: there must be 1 or 2 curves, you asked for", numcurves;
         stop=1 # flag to halt in END 
         exit 1 # stop reading input and go to END
      }
   for ( curve = 0; curve < numcurves; curve++ ) {
      getline curvename[curve];
      for (point = 0; point < numpts; point++) {
         getline; xlabel[point] = $1; data[curve,point] = $2;
      }
   }
   exit
}   

END {
   if (0 != stop) { exit stop }
   print "\nnewgraph\n  title :", title
   print "  yaxis  min 0 max 1 label : ", ylabel;
   for ( curve = 0; curve < numcurves; curve++ ) {
      print "  newcurve\n    label :", curvename[curve]
      print "    marktype xbar " patt[curve] "\n"\
            "    marksize", width "\n"           \
            "    pts"
      inc = (2 == numcurves) ?(gap + 2 * width) :(width + gap);
      pos = 2 * gap
      if (1 == curve) { pos += width }
      for (point = 0; point < numpts; point++) {
         printf("     %5.1f %s\n", pos, data[curve,point])
         pos += inc
      }
   }

   printf("  xaxis  min 0 max %5.1f\n", pos - inc + width);
   print "    no_auto_hash_marks no_draw_axis_label no_auto_hash_labels"
   print "    hash_labels rotate -90 vjt hjl"
   pos = (1 == numcurves) ?(gap + 0.5 * width) :(gap + width)
   inc = (1 == numcurves) ?(gap + width) :(gap + 2 * width)
   for(point = 0; point < numpts; point++) {
      printf("      hash_label at %5.1f : %s\n", pos, xlabel[point])
      pos += inc
   }
}
